import CoolProp.CoolProp as CP
import numpy as np
import scipy.optimize as solver

# ~ # hlavni parametry -> z reseni
# ~ T_sat = 167.0 # C
# ~ m_kond = 4.59e-3 # kg/s

# ~ m_cool0 = 1. # kg/s
# ~ T_cool_in = 25. # C
# ~ k_tr = 15. # W/m.K

# ~ # geometrie
# ~ D = 0.016 # m
# ~ tl = 0.003
# ~ d_cool = 0.25

# ~ power = 9501.5 # W

# ~ # hmotnistní podíly páry a plynu
# ~ w_g = 0.15 # plyn
# ~ w_w = 0.85 # voda

def Condensation(T_sat, m_kond, P_cool, T_cool_in, m_cool_guess, d, tl, D, w_g, Power, k_tr, dT_w, gas):
    """
    T_sat - saturační teplota sekundární strany [C]
    m_kond - parní výkon sekundární strany [kg/s]
    d - průměr trubky chladícího okruhu [m]
    D - průměr terciálního okruhu [m]
    w_w - hmotnostní koncentrace páry [kg/kg]
    w_g - hmotnostní koncentrace Argonu [kg/kg]
    """
    
    CP.apply_simple_mixing_rule(gas, 'Water', 'linear')
    CP.set_config_bool(CP.OVERWRITE_BINARY_INTERACTION, True)
    
    # molární hmotnosti
    if gas == "Argon":
        M_g = 39.948 # g/mol
    elif gas == "Nitrogen":
        M_g = 28.
    elif gas == "Air":
        M_g = 28.
    elif gas == "Helium":
        M_g = 4.003
    M_w = 18.01528 # g/mol
    
    P_sat = CP.PropsSI('P','T',T_sat+273.15,'Q', 0, 'H2O') # saturacni tlak
    
    # počet uzlů III.O.
    n_node = 10
    
    # geometrie anuloidu
    L = np.pi*D
    dL = L/(n_node-1)
    d_in = d - 2*tl
    d_str = (d_in+d)/2
    S = 4 * np.pi * (D/2) * (d_str/2)
    dS = S/(n_node-1)
    A_in = np.pi * d_in**2 / 4
    
    # minimalni koncentrace páry ######
    test_pass = False
    w_w_test = 0.4
    print("Hledání minimální koncentrace páry ...")
    while test_pass == False:
        w_g_test = 1.0 - w_w_test
        # molární podíly
        x_g_test = (w_g_test/M_g)/(w_g_test/M_g + w_w_test/M_w)
        x_w_test = (w_w_test/M_w)/(w_g_test/M_g + w_w_test/M_w)
        # parciální tlak
        P_g_test = P_sat*x_g_test
        P_w_test = P_sat*x_w_test
        T_wb_test = CP.PropsSI('T','P',P_w_test,'Q', 1, 'H2O')-273.15
        # teplota stěny
        T_w_test = T_wb_test - dT_w
        # interface
        T_i_test = T_w_test + 0.25*dT_w
        # parcialni tlak vody na interface
        P_wi_test = CP.PropsSI('P','T',T_i_test+273.15,'Q', 0, 'H2O')
        P_gi_test = P_sat - P_wi_test
        # koncentrace na interface
        x_wi_test = P_wi_test/P_sat
        x_gi_test = P_gi_test/P_sat
        ###
        pair_and_fractions_i_test = gas+'['+str(x_gi_test)+']&Water['+str(x_wi_test)+']'
        try:
            Rho_test_tabulek = CP.PropsSI('Dmass','T',T_i_test+273.15,'P',P_sat, pair_and_fractions_i_test)
            test_pass = True
            w_w_min = int(w_w_test*100)/100
            w_g_max = 1.-w_w_min
            P_g_0_test = P_g_test*293.15/(T_sat+273.15)
            print("Minimální koncentrace vodní páry:", w_w_min, "Tlak plynu ve studeném stavu:", round(P_g_0_test/1e5,2), "bar ("+gas+")")
        except:
            w_w_test += .01
        
        if w_w_test > 1.0:
            w_g_max = 0.
            break
    
    if w_g > w_g_max:
        w_g = w_g_max
    
    # hmotnostní koncentrace
    w_w = 1. - w_g

    # molární podíly
    x_g = (w_g/M_g)/(w_g/M_g + w_w/M_w)
    x_w = (w_w/M_w)/(w_g/M_g + w_w/M_w)
    
    # kolizní rozměry
    sigma_1 = 2.655
    sigma_2 = 3.432
    sigma_12 = (sigma_1 + sigma_2)/2
    
    # Molekulární energetický potenciál
    epsilon_k_1 = 809.1
    epsilon_k_2 = 122.4
    epsilon_k_12 = (epsilon_k_1 * epsilon_k_2)**.5
    
    Boltzman = 1.38065e-23 # J/K
    
    epsilon_12 = epsilon_k_12 * Boltzman

    # parciální tlak
    P_g = P_sat*x_g
    P_w = P_sat*x_w
    
    P_g_0 = P_g*293.15/(T_sat+273.15)

    T_wb = CP.PropsSI('T','P',P_w,'Q', 1, 'H2O')-273.15
    
    # teplota stěny
    T_w = T_wb - dT_w
    
    # interface
    T_i = T_w + 0.25*dT_w
    
    # teplota kondenzátu
    T_cond = (T_w + T_i)/2
    
    # průměrná logaritmická teplota
    T_ave = (T_wb - T_i)/np.log((T_wb+273.15)/(T_i+273.15))
    
    # redukovaná teplota
    T_12_o = (T_wb+273.15)*Boltzman / epsilon_12
    
    # kolizní integrál
    Omega_11_T12o = 1.06036/T_12_o**0.1561 + 0.193/np.exp(0.47635*T_12_o) + 1.03587/np.exp(1.52996*T_12_o) + 1.76474/np.exp(3.89411*T_12_o)
    
    # parcialni tlak vody na interface
    P_wi = CP.PropsSI('P','T',T_i+273.15,'Q', 0, 'H2O')
    P_gi = P_sat - P_wi
    
    # koncentrace na interface
    x_wi = P_wi/P_sat
    x_gi = P_gi/P_sat
    
    # střední logaritmická koncentrace
    # ~ x_g_ave = (x_g-x_gi)/np.log(x_g/x_gi)
    # ~ Phi_x_g = x_gi/x_g_ave

    w_wi = (x_wi*M_w)/(x_wi*M_w + x_gi*M_g)
    w_gi = (x_gi*M_w)/(x_wi*M_w + x_gi*M_g)
    
    # průměrná logaritmická koncentrace
    Phi = - np.log((1-x_g)/(1-x_gi)) / np.log((x_g)/(x_gi))
    
    # univerzální plynová konstanta
    R = 8.3144598 # J/mol.K
    
    # inicializace smesi a smesovaci pravidlo pro CoolProp
    pair_and_fractions_b = gas+'['+str(x_g)+']&Water['+str(x_w)+']'
    pair_and_fractions_i = gas+'['+str(x_gi)+']&Water['+str(x_wi)+']'
    
    # vlastnosti smesi na interface a v prostoru + prumer
    try:
        Rho_mix_b = CP.PropsSI('Dmass','T',T_sat+273.15,'P',P_sat, pair_and_fractions_b)
        Rho_mix_i = CP.PropsSI('Dmass','T',T_i+273.15,'P',P_sat, pair_and_fractions_i)
        Rho_mix = (Rho_mix_b + Rho_mix_i)/2

        k_mix_b = CP.PropsSI('L','T',T_sat+273.15,'P',P_sat, pair_and_fractions_b)
        k_mix_i = CP.PropsSI('L','T',T_i+273.15,'P',P_sat, pair_and_fractions_i)
        k_mix = (k_mix_b + k_mix_i)/2

        c_mix_b = CP.PropsSI('C','T',T_sat+273.15,'P',P_sat, pair_and_fractions_b)
        c_mix_i = CP.PropsSI('C','T',T_i+273.15,'P',P_sat, pair_and_fractions_i)
        c_mix = (c_mix_b + c_mix_i)/2

        nu_mix_b = CP.PropsSI('V','T',T_sat+273.15,'P',P_sat, pair_and_fractions_b)
        nu_mix_i = CP.PropsSI('V','T',T_i+273.15,'P',P_sat, pair_and_fractions_i)
        nu_mix = (nu_mix_b + nu_mix_i)/2
    except:
        return float(T_cond), None, None, None, None, None, P_g_0, w_g_max, w_g, False
    
    # podobnostní čísla
    # ~ beta = 2/(T_sat+T_i)
    # ~ Gr = beta * (T_sat-T_i) * 9.81 * d**3 / (nu_mix / Rho_mix)**2
    Gr = 9.81 * Rho_mix_b * d**3 * (Rho_mix_i - Rho_mix_b) / nu_mix**2
    # ~ Pr = (nu_mix * c_mix) / k_mix
    # ~ Ra = Gr*Pr
    # ~ Nu_D = (0.6 + 0.387*Ra**(1/6)/(1+(0.559/Pr)**(9/16))**(8/27))**2

    # ~ # součinitel přestupu tepla konvekce trubky
    # ~ h_D = Nu_D*k_mix/d

    # filmová složka součinitele přestupu tepla
    Rho_cond_g = CP.PropsSI('D','T',T_i+273.15,'Q', 1, 'H2O')
    h_cond_g = CP.PropsSI('H','T',T_i+273.15,'Q', 1, 'H2O')

    Rho_cond_l = CP.PropsSI('D','T',T_i+273.15,'Q', 0, 'H2O')
    h_cond_l = CP.PropsSI('H','T',T_i+273.15,'Q', 0, 'H2O')
    k_cond_l = CP.PropsSI('L','T',T_i+273.15,'Q', 0, 'H2O')
    nu_cond_l = CP.PropsSI('V','T',T_i+273.15,'Q', 0, 'H2O')
    c_cond_l = CP.PropsSI('C','T',T_i+273.15,'Q', 0, 'H2O')

    l_cond = h_cond_g - h_cond_l
    
    h_F = 0.725 * (9.81 * Rho_cond_l * (Rho_cond_l-Rho_cond_g) * k_cond_l**3 * l_cond / (d * nu_cond_l * (T_i-T_w) ))**0.25
    
    D_12 = 0.0018583 * ((T_sat+273.15)**3 * (1/M_g + 1/M_w))**.5 * (P_sat/1e5 * sigma_12**2 * Omega_11_T12o)**-1 # cm2/s
    D_12 /= 1e4
    
    # kondenzační tepelná vodivost
    k_paterson = 1/Phi * (l_cond**2 * P_sat * (M_w/1e3)**2 * D_12 / R**2 / T_ave**3)
    
    # Schmidtovo číslo
    Sc = nu_mix / Rho_mix / D_12
    
    # Sherwoodovo číslo
    Sh = 0.48*(Gr*Sc)**0.25
    # ~ Sh_star = Sh * Phi_x_g
    
    h_kond = Sh*k_paterson/d
    
    h_celk = 1/(1/h_F + 1/h_kond)
    
    H_cool_in = CP.PropsSI('H','T', T_cool_in+273.15,'P', P_cool, 'H2O')
    
    def m_cool(m_cool):
        
        if m_cool > 0:
            # vlastnosti chladiva
            dH_cool = Power/m_cool
            
            H_cool_out = H_cool_in + dH_cool
            global T_cool_out, x_cool_out
            T_cool_out = CP.PropsSI('T', 'P', P_cool, 'H', H_cool_out, 'H2O') - 273.15
            x_cool_out = CP.PropsSI('Q', 'P', P_cool, 'H', H_cool_out, 'H2O')
            
            H_cool = np.linspace(H_cool_in, H_cool_out, num=n_node)
            T_cool = CP.PropsSI('T', 'P', P_cool, 'H', H_cool, 'H2O') - 273.15
            Rho_cool = CP.PropsSI('D', 'P', P_cool, 'H', H_cool, 'H2O')
            
            c_cool = m_cool / (A_in * Rho_cool)
            
            global C_cool
            C_cool = np.average(c_cool)
            
            Pr_cool = CP.PropsSI('Prandtl', 'P', P_cool, 'H', H_cool, 'H2O')
            Re_cool = Rho_cool * c_cool * d_in / CP.PropsSI('V', 'P', P_cool, 'H', H_cool, 'H2O')
            Nu_cool = 0.023 * Re_cool**0.8 * Pr_cool**0.4
            h_cool = Nu_cool * CP.PropsSI('L', 'P', P_cool, 'H', H_cool, 'H2O') / d_in
            
            # prostup tepla válcovou stěnou
            b = 2*np.pi*k_tr*(T_i-T_cool)
            a1 = k_tr/(d_in/2)/h_cool
            a2 = k_tr/(d/2)/h_celk
            c = np.log((d/2)/(d_in/2))
            Q = b / (a1 + a2 + c) * dL
            Q[0] /= 2
            Q[-1] /= 2
            P_transf = sum(Q)
            
            err = abs(Power-P_transf)
            
            print("P0:", Power, "W; P1:", P_transf, "W; diff:", err, "W; ---- m:", m_cool, "kg/s c:", C_cool, "m/s; x:", x_cool_out)
            
            return(err)
        else:
            return(1e6)
    
    res = solver.minimize(m_cool, m_cool_guess, method='Nelder-Mead', bounds=(0.001,5) ,tol=1e-1)
    m_cool = res.x
    success = res.success
    # ~ print(success)
    
    return float(T_cond), T_cool_out, float(h_celk), float(m_cool), C_cool, float(x_cool_out), P_g_0, w_g_max, w_g, success

# ~ print(Condensation(T_sat, m_kond, 8e5, T_cool_in, 1., D, tl, d_cool, w_g, power, k_tr, 5., 'Nitrogen'))
